/*********************************************************************

Procedures to determine the auto-covariance, auto-correlation and
pitch estimates.

*********************************************************************/

/*********************************************************************

autoCovariance: calculate the auto-covariance of a window

*********************************************************************/

double * autoCovariance(double * samples, int window, int maxK, int offset){

 double * out = (double *) malloc((maxK+1) * sizeof(double));
 int i = 0, k = 0;
 double * buffer = (double *) malloc(sizeof(double) * (window+maxK));

 if (out){

  for (i = 0; i < window + maxK; i++)
   buffer[i] = samples[offset+i];

  for (k = 0; k <= maxK; k++){

   out[k] = 0.0;

   for (i = 0; i <= window-1; i++){

    out[k] += buffer[i] * buffer[i+k];

   } /* i */

   out[k] /= window;

  } /* k */
 
 } /* if */

 free(buffer);
 return out;

}


/*********************************************************************

normaliseCovariance: normalise the auto-covariance, i.e return the
auto-correlation

*********************************************************************/

double * normaliseCovariance(double * covariance, int k){

 double * out = (double *) malloc((k+1) * sizeof(double));
 int i = 0;

 if (covariance[0]){

 if (out){

  for (i = k; i >= 0; i--) out[i] = covariance[i] / covariance[0];

 } /* if */

 } else printf("C(0) is zero, can't normalise\n");
 return out;

}

/*********************************************************************

pitchEstimate: pitch estimate by auto-correlation method

*********************************************************************/

double pitchEstimate(double * correlation, int maxK, double triggerThreshold, double resetThreshold){

 int i = 0;
 int mode = PITCH_RESET; /* wait for normalised signal to drop below reset */
 int intervals = 1;
 int * indices = (int *) malloc(sizeof(int));

 double average = -1.0; 

 indices[0] = 0;

 for (i = 0; i < maxK; i++){

  if (mode == PITCH_RESET && correlation[i] < resetThreshold){

   mode = PITCH_TRIGGER;
   continue;

  }

  if (mode == PITCH_TRIGGER && correlation[i] > triggerThreshold){

   /* Trigger Event */

   indices = (int *) realloc(indices, ++intervals * sizeof(int));
   indices[intervals-1] = i;

   mode = PITCH_RESET;
   continue;

  }

 } /* i */

 if (intervals > 1){
  average = 0;
  for (i = 1; i < intervals; i++)
   average += (double) indices[i] - (double) indices[i-1];
  average /= (double) (intervals - 1);
 }

 free(indices);

 return average;

}

