/*********************************************************************

durbin: drive the main recursion (iteration) of Durbin's algorithm

*********************************************************************/

double durbin(double * a, double * window, int N, int p){

 double * k     = (double *) malloc((p+1) * sizeof(double));
 double * old_a = (double *) malloc((p+1) * sizeof(double));
 double * new_a = (double *) malloc((p+1) * sizeof(double));

 int j = 0, m = 0;

 double E = 0.0;

 /* Begin */

 E = R(window, N, 0);
 k[1] = R(window, N, 1)/E;
 old_a[1] = k[1];

 E *= (1 - k[1] * k[1]);

 for (j = 2; j <= p; j++){

  k[j] = parcor(window,N,j,E,old_a);

  new_a[j] = k[j];

  for (m = 1; m < j; m++) /* Determine all a's for this iteration */
   new_a[m] = old_a[m] - k[j] * old_a[j-m];

  E *= (1 - k[j] * k[j]);

  for (m = 1; m <= j; m++) old_a[m] = new_a[m];

 }

 for (m = 1; m <= p; m++) a[m] = new_a[m];

 /* End */

 free(old_a);
 free(new_a);
 free(k);

 return E;

}


/*********************************************************************

parcor: determine the partial correlation coefficients for durbin

*********************************************************************/

double parcor(double * window, int N, int i, double E, double * a){

 double k = 0.0;

 int j = 0;

 for (j = 1; j <= i - 1; j++)
  k += a[j] * R(window, N, i - j);

 return (R(window, N, i) - k) / E;

}


/*********************************************************************

R: determine the auto-correlation for durbin

*********************************************************************/

double R(double * window, int N, int k){

 int i;
 double r = 0.0;

 for (i = 0; i <= N - k - 1; i++)
  r += window[i] * window[i+k];

 return r;

}


