/* Read a Magpie clipboard dump and turn it into a Houdini-readable list of indices */


#include <stdio.h>
#include <string.h>


#define DEFAULT_TARGET	"Closed"
#define PHONEMES	"magpie.phonemes"


void addIndex(int ** data, int *indices, int index);
int readLine(FILE * filePtr, char * buffer, int bufsize);
int getPhonemes(char * filename);


char id[64][64] = {'\0'}; /* Can have 64 x 63 character identifiers */
int identifiers = 0;


int main(int argc, char ** argv){

 FILE * infile = NULL;
 FILE * outfile = NULL;
 
 char * token;
 
 char filename[512];
 char * buffer = (char *) malloc(sizeof(char) * 1024);
 
 int status = -1;
 int i = 0;
 int exists = 0;
 int indices = 0;
 int index = 0;
 int pfile = 0; /* phoneme file has been opened */

 int * data = NULL;
 
 if (argc < 2){
 
  printf("Usage: %s <magpie file>\n",argv[0]);
  exit(1);
  
 }


/* Get phoneme file, if any */

 pfile = getPhonemes(PHONEMES);

/*
 Read file, make array or data structs or whatever, prompt user for
 index map, store identifiers as indices into id array
*/

 if ((infile = fopen(argv[1],"r")) != NULL){

  while ((status = readLine(infile,buffer,1024)) != -1){
  
   if (status > 0){
    
    /* tokenize: <sample> <timecode> <target char/str or maybe not> */

    token = strtok(buffer,"\t ");
    index = atoi(token); /* or just increment i */
    
    /* time index */
   
    token = strtok(NULL,"\t ");
   
    /* timecode */
   
    token = strtok(NULL,"\t ");
   
    /* target identifier, or not */
   
    if (token == NULL || token[0] == 13){ /* hold last or DEFAULT_TARGET if no previous */
    
     addIndex(&data,&indices,-1);
    
    } else { /* Have an identifier */

     if (identifiers == 0){

      /* Can't be here if phoneme file was read successfully */
      /* store index 0 in data array */

       strcpy(id[identifiers++],token);
       addIndex(&data,&indices,0);

     } else { /* Search id array for token */
    
      exists = 0;
      
      for (i = 0; i < identifiers; i++){
      
       if (!strcmp(id[i],token)){
       
        /* store index i in data array*/
        addIndex(&data,&indices,i);
	exists = 1;
        break;
	
       }
 
      } /* i */
    
      if (!exists){ /* Add token to id array and store index in data array */

       if (pfile == 0){     
        strcpy(id[identifiers++],token);
        /* store index identifiers-1 in array */
        addIndex(&data,&indices,identifiers-1);
       } else printf("Failed to match identifier \"%s\"\n",token);
  
      } /* if !exists */
    
     } /* if identifiers == 0 */
    
    } /* if token == NULL */
   
   } /* else line is empty, don't bother*/
  
  } /* while */
  
  printf("Done reading file\n");
  
  if (pfile == 0)
   for (i = 0; i < identifiers; i++) printf("id[%d]: %s\n",i,id[i]);
  
  sprintf(filename,"%s.clip",argv[1]);
  printf("Writing file \"%s\"\n",filename);
  outfile = fopen(filename,"w");
  
  fprintf(outfile,"{\n   rate = %d\n   start = 0\n   tracklength = %d\n   tracks = 1\n   {\n",25,indices);
  fprintf(outfile,"      name = magpie\n      data = ");

  for (i = 0; i < indices; i++){
  
   fprintf(outfile,"%d ",data[i]);

  }

  fprintf(outfile,"\n   }\n}\n");
  
  fclose(outfile);
  
 } else {
 
  printf("Failed to open file \"%s\"\n",argv[1]);
  exit(1);
  
 } /* fopen */


 return 0;

}




void addIndex(int ** data, int *indices, int index){
  
 *data = (int *) realloc(*data,((*indices)+1)*sizeof(int));

 if (index == -1){ /* No id token, hold last or else DEFAULT_TARGET */
 
  if (*indices == 0){ /* DEFAULT_TARGET */

   strcpy(id[identifiers++],DEFAULT_TARGET);
   (*data)[0] = 0;
  
  } else {
  
   (*data)[*indices] = (*data)[*indices-1];
 
  } /* if *indices */
 
 } else {

  (*data)[*indices] = index;

 }
 
 *indices = *indices + 1;
  
}



int getPhonemes(char * filename){

 FILE * file;
 char buffer[1024];
 int length = -1;

 if ((file = fopen(filename,"r")) == NULL){

  printf("Failed to read phoneme file, using dynamic method\n");
  return 0;

 } else {

  printf("Phoneme file opened\n");
  while((length = readLine(file,buffer,1024)) != -1){

   if (length > 0){

    strcpy(id[identifiers++],buffer);
    /* printf("%d: \"%s\"\n",identifiers,id[identifiers-1nedit ]); */

   }

  }

  fclose(file);

  printf("Read %d targets\n",identifiers);

  return identifiers;

 }

}



int readLine(FILE * filePtr, char * buffer, int bufsize){

 int i = 0;
 char tmp = fgetc(filePtr);

  while (!feof(filePtr) &&
         i < bufsize &&
         tmp != 10){

   /* printf("%d: %c (%d)\n",i,tmp,(int)tmp); */
 
   buffer[i++] = tmp;

   tmp = fgetc(filePtr);
  }

 /* Null-terminate the string */

 buffer[i] = '\0';

 if (feof(filePtr)) return -1;
 else return i;

}

