//
// Take a clip file and do a comparision with templates at each sample
//
//
//


#define MAX_TARGETS 	32
#define DFT_SAMPLES 	32

#define OUT_EUCLID	"_euclid.clip"
#define OUT_SPECTRUM	"_spectrum.clip"

#include <stdlib.h>
#include <iostream.h>
#include <fstream.h>
#include <string.h>

#include "CPD_Clip.h"	// Houdini .clip library
#include "Spectrum.h"	// DFT stuff


int doMatch(char * filename, double * targets, int n);
int loadTarget(char * name, double ** targets, int n);


int n = 0; // number of targets


int main(int argc, char ** argv){

 double * targets = NULL;

 char input[512];
 char filename[512];

 ifstream myfile;

 CPD_Clip myClip, outClip;


 cout.setf(ios::stdio);


 cout << "File to test: ";
 cin >> filename;

// Do something with input

 cout << "Target " << (n+1) << " (EOF to end): ";
 cin >> input;

 while (!cin.eof()){

  // Do something with target n
  // need to avarage each channel

  int status = loadTarget(input, &targets, n+1);

  if (!status) n++; else cout << "Failed to read file \"" << input << "\"\n";

  cout << "Target " << (n+1) << ": ";
  cin >> input;

 }

 cout << "\n\nFinal target count: " << n << "\n";

 int status = doMatch(filename, targets, n);
 

 return 0;

} // main


int loadTarget(char * name, double ** targets, int n){

  ifstream thisTarget(name,ios::in,filebuf::openprot);

  if (!thisTarget.fail()){

   CPD_Clip thisClip;
   thisClip.load(thisTarget);
   thisTarget.close();

   // Should now have clip loaded

//   if (n == 1){ // Make index 0 template
//
//    *targets = (double *) realloc(*targets,sizeof(double)*thisClip.number_tracks);
//    for (int i = 0; i < thisClip.number_tracks; i++) (*targets)[i] = 0.0;
//    
//   }

   double * a = new double[thisClip.number_tracks]; // One average per track

   for (int i = 0; i < thisClip.track_length - 1; i++){

    for (int j = 0; j < thisClip.number_tracks; j++)
     a[j] += thisClip.tracks[j].data[i];

   }

   for (int j = 0; j < thisClip.number_tracks; j++){
    a[j] /= (thisClip.track_length - 1);
    printf("\ta%d:\t%f\n",j,a[j]);
   }

   // Now stick them in targets array

   *targets = (double *) realloc(*targets,sizeof(double)*n*thisClip.number_tracks);
   memcpy(*targets+(n-1)*thisClip.number_tracks,a,thisClip.number_tracks*sizeof(double));
   
   return 0; // ok

  } else return -1; // failed to open stream

}


int doMatch(char * filename, double * targets, int n){

 int i;

// Load file

  ifstream thisTarget(filename,ios::in,filebuf::openprot);

  if (!thisTarget.fail()){

   CPD_Clip thisClip;
   thisClip.load(thisTarget);
   thisTarget.close();

// Shortest distance between parameter sets

   double * indices_euclid = new double[thisClip.track_length];

   printf("Calculating shortest distance (%d) ", thisClip.track_length);

   for (i = 0; i < thisClip.track_length; i++){

    int minIndex = -1.0;
    double minDistance = -1.0;

    for (int j = 0; j < n; j++){

     double sum = 0.0;

     for (int k = 0; k < thisClip.number_tracks; k++){

      sum += pow(thisClip.tracks[k].data[i] - targets[(j*n)+k], 2.0);

     } // k

     sum = sqrt(sum);

     if (sum < minDistance || minDistance == -1.0){

      minDistance = sum;
      minIndex = j;

     }

   } // j

   indices_euclid[i] = (double) minIndex + 1.0;

    if (i % 100 == 99) printf(".");

    fflush(stdout);
 
   } // i

   printf("\n");


// Shortest distance between spectra (if it makes any sense)

   double * indices_spectrum = new double[thisClip.track_length];
   double * tmp = new double[thisClip.number_tracks];
 
   printf("Calculating spectral distance ");

   for (i = 0; i < thisClip.track_length; i++){

    for (int l = 0; l < thisClip.number_tracks; l++)
     tmp[l] = thisClip.tracks[l].data[i];

    Spectrum tmpSpec(tmp,thisClip.number_tracks,DFT_SAMPLES);

    int minIndex = -1;
    double minDistance = -1.0;

    for (int j = 0; j < n; j++){

     Spectrum targetSpec(targets+(j*n),thisClip.number_tracks,DFT_SAMPLES);
     double sum = tmpSpec - targetSpec;

     if (sum < minDistance || minDistance == -1.0){

      minDistance = sum;
      minIndex = j + 1;

     }

    } // j

    indices_spectrum[i] = (double) minIndex;
    if (i % 100 == 99) printf(".");
    fflush(stdout);

   } // i

   printf("\n");

// return array or save indices or something (save as clip, for example)

// Save euclidean clip

 CPD_Clip outClip;
 outClip.addTrack("euclid",indices_euclid,thisClip.track_length,"hold","hold",0);

 char * outname = new char[strlen(filename) + strlen(OUT_EUCLID) + 1];
 sprintf(outname,"%s%s",filename,OUT_EUCLID);
 cout << "Saving Euclidean indices (\"" << outname << "\")\n";
 ofstream outStream(outname);
 outClip.save(outStream);
 outStream.close();

 FILE * testFile = fopen(outname,"w");
 for (i = 0; i < thisClip.track_length; i++){

  fprintf(testFile,"%d\n",(int)indices_euclid[i]);

 }
 fclose(testFile);

 delete [] outname;

// outClip.reset();
// outClip.addTrack("spectrum",indices_spectrum,thisClip.number_tracks,"hold","hold",0);

 outname = new char[strlen(filename) + strlen(OUT_SPECTRUM) + 1];
 sprintf(outname,"%s%s",filename,OUT_SPECTRUM);

 cout << "Saving Spectrum indices (\"" << outname << "\")\n";

// outStream.open(outname);
// outClip.save(outStream);
// outStream.close();

 testFile = fopen(outname,"w");
 for (i = 0; i < thisClip.track_length; i++){

  fprintf(testFile,"%d\n",(int)indices_spectrum[i]);

 }
 fclose(testFile);

 delete [] outname;

 cout << "All done\n";

 return 0;

 } else return -1;

}




